@@ -1,6 +1,11 @@
 """Package for working with Cypress Creek Renewables Dashboard process
 """
 print('Starting imports')
+import warnings
+from pandas.errors import PerformanceWarning
+warnings.simplefilter(action='ignore', category=(FutureWarning, PerformanceWarning))
+warnings.filterwarnings(action='ignore', message='divide by zero')
+
 from ccrenew.dashboard.data_processing import CCR as ccr
 
 print('Pulling df_keys from Smartsheets...')
