@@ -92,8 +92,6 @@ def calculate_inverter_availability_and_loss(df , df_Pvsyst, P_exp):
     aux_prev = aux['Inv_CUM_OFF_2'].shift(1)
     aux_next = aux['Inv_CUM_OFF_2'].shift(-1)
     bad_indices = list((aux_curr == 0) & (aux_prev == 1) & (aux_next == 1))
-    if sum(bad_indices) > 0:
-        print("*** Removed", sum(bad_indices), "1-hour rows in the middle of an outage")
     aux.loc[bad_indices, 'Inv_CUM_OFF_4'] = 0
     
     #
@@ -312,8 +310,6 @@ def calculate_inverter_availability_and_loss(df , df_Pvsyst, P_exp):
     ava_curr = ava['ALL_OFF']
     ava_prev = ava['ALL_OFF'].shift(1)
     ava_next = ava['ALL_OFF'].shift(-1)
-    if sum(bad_indices) > 0:
-        print("*** Removed", sum(bad_indices), "1-hour rows in the middle of an outage")
     bad_indices = list((ava_curr == 0) & (ava_prev == 1) & (ava_next == 1))
 
     ava.loc[bad_indices, 'ALL_OFF_3'] = 0
