@@ -7,6 +7,8 @@ import math
 import s3fs
 import shapely.geometry as gs
 
+from ccrenew.dashboard.utils.project_neighbors import haversine
+
 """
 Imports above docstring to exclude from Sphinx AutoAPI
 Created on Fri May 26 11:43:57 2017
@@ -16,27 +18,28 @@ Created on Fri May 26 11:43:57 2017
 @author: BLUMENTHAL
 """
 
-def timeseries(orig_df, df_index, lat, lon):
+def timeseries(raw_snow_df, project_df_index, project_lat, project_lon):
 
-    X, Y = np.meshgrid([lon],[lat])
-    df = orig_df.copy()
+    X, Y = np.meshgrid([project_lon],[project_lat])
+    df = raw_snow_df.copy()
     
     #Z = []
     
     #filter sites based on distance to project
-    df['dist'] = ((df.Longitude - lon).pow(2) + (df.Latitude - lat).pow(2)).pow(0.5) * 69
+    df['dist'] = ((df.Longitude - project_lon).pow(2) + (df.Latitude - project_lat).pow(2)).pow(0.5) * 69
+    # df['dist'] = haversine(project_lon, project_lat, df['Longitude'], df['Latitude'])
     df = df.loc[ df['dist'] < 200, : ]
     
     #filter unused columns via production data index    
     # date_cols = [x for x in orig_df.columns if (type(x) in [datetime.datetime, pd.Timestamp])]
     date_cols = []
-    for col in orig_df.columns:
+    for col in raw_snow_df.columns:
         try:
             pd.to_datetime(col)
             date_cols.append(col)
         except:
             continue
-    df_index_dates = pd.to_datetime(list(set([x.date() for x in df_index])))
+    df_index_dates = pd.to_datetime(list(set([x.date() for x in project_df_index])))
 
     
     #loop through applicable snow data columns
@@ -44,7 +47,7 @@ def timeseries(orig_df, df_index, lat, lon):
     
     #z_df = pd.DataFrame(index = match_dates, columns = ['snow']) 
     #create a snow df that is empty, to fill later
-    z_df = pd.DataFrame(index = df_index, columns = ['snow'])
+    z_df = pd.DataFrame(index = project_df_index, columns = ['snow'])
     z_df['snow'] = 0
 
     for col in match_dates:
@@ -69,7 +72,7 @@ def timeseries(orig_df, df_index, lat, lon):
         
         #before interpolating, check to see if point is within mesh. If not then use closest point
         hull = gs.MultiPoint(cartcoord).convex_hull
-        if hull.contains(gs.Point([lon, lat])):
+        if hull.contains(gs.Point([project_lon, project_lat])):
             #print('contains')
             interp = scipy.interpolate.LinearNDInterpolator(cartcoord, z, fill_value= 0)
             #print(interp)
