@@ -6,7 +6,10 @@ import pandas as pd
 import requests
 import s3fs
 
-import ccrenew.dashboard.data_processing.CCR as ccr
+from ccrenew.dashboard import (
+    ccr,
+    all_df_keys
+)
 
 """
 Imports above docstring to exclude from Sphinx AutoAPI
@@ -25,40 +28,42 @@ s3_config = boto3.client('s3')
 bucket = 'perfdatadev.ccrenew.com'
 bucket_prefix = "5min_archive/Solcats/"   #new folder for salesforce
 
-df_keys = ccr.all_df_keys
+df_keys = all_df_keys
 
 # https://preview-docs.solcast.com.au/
 
 
 
 ####################################################################################################
-def get_df(CCR_ID, start, end,prod_data=False):
-    """Function that actually retrieves the S3 data each time. 
-        It references the Global 'CCR_ID' and takes start and end
-        dates as Args
-    Args:
-        start (str) or (datetime) of date range for data pull
-        end (str) or (datetime) of date range for data pull
-    Returns: 
-        df (dataframe) of S3 data and sets global df variable with data
-    """    
-    if prod_data:
-        try:
-            df = get_all_fields(CCR_ID,start,end,prod_data)
-            print("\nGot it!")
+# def get_df(CCR_ID, start, end,prod_data=False):
+#     """Function that actually retrieves the S3 data each time. 
+#         It references the Global 'CCR_ID' and takes start and end
+#         dates as Args
+#     Args:
+#         start (str) or (datetime) of date range for data pull
+#         end (str) or (datetime) of date range for data pull
+#     Returns: 
+#         df (dataframe) of S3 data and sets global df variable with data
+#     """    
+#     if prod_data:
+#         try:
+#             df = get_all_fields(CCR_ID,start,end,prod_data)
+#             print("\nGot it!")
                    
-        except Exception as e:
-            print("Error...\n{}".format(e))
+#         except Exception as e:
+#             ("Error...\n{}".format(e))
     
-        return df
-    else:
-        try:
-            df = get_all_fields(CCR_ID,start,end)
-            print("\nGot it!")
+#         return df
+#     else:
+#         try:
+#             df = get_all_fields(CCR_ID,start,end)
+#             print("\nGot it!")
                
-        except Exception as e:
-            print("Error...\n{}".format(e))
-def get_all_fields(CCR_ID, st, ed, prod_data=False,pool_size = 6):
+#         except Exception as e:
+#             print("Error...\n{}".format(e))
+
+
+def get_df(CCR_ID, st, ed, prod_data=False,pool_size = 6):
     """
     project_name: string to match with df_keys, eg 'Hardison Farm'
     st, ed: datetime strings or datetime objects, eg '2018-01-01'
@@ -114,12 +119,12 @@ def retrieve_df(key):
     
     fs = s3fs.S3FileSystem(anon=False)
     path = "s3://{b}/{k}".format(b=bucket, k=key)
-    with fs.open(path, 'rb') as f:
-        try:
+    try:
+        with fs.open(path, 'rb') as f:
             df = pd.read_csv(f, index_col = 0, parse_dates = True)
             df = df.loc[~df.index.duplicated(), :]
-        except s3fs.core.FileNotFoundError:
-            return pd.DataFrame()
+    except FileNotFoundError:
+        df = pd.DataFrame()
     return df
 
 
