@@ -1,26 +1,24 @@
 # -*- coding: utf-8 -*-
 
 from ccrenew.dashboard.utils.logging_conf import get_logging_config
+from collections.abc import Iterable
 from datetime import datetime
 from dateutil import parser
 import logging
 import logging.config
 from matplotlib.pyplot import waitforbuttonpress
-from numbers import Number
 import numpy as np
 import os
 import pandas as pd
-from pandas import DataFrame
 from pandas.io.sql import SQLTable
 import pickle
-import scipy.stats as sct
-import shutil
 import sys
 import time
 import traceback
-from typing import Type, Union
+from typing import Union
 import warnings
 
+from ccrenew import __version__
 from ccrenew.dashboard import (
     ccr,
     all_df_keys,
@@ -31,6 +29,9 @@ from ccrenew.dashboard.utils.df_tools import df_update_join
 import ccrenew.dashboard.utils.dashboard_utils as utils
 import ccrenew.dashboard.utils.project_neighbors as neighbs
 
+from ccrenew.dashboard.data_processing.batch_process import (
+    export_pool,
+    process_pool)
     
 # Python 2 compatibility
 if sys.version_info.major == 3:
@@ -50,8 +51,8 @@ LOGGING_CONFIG = get_logging_config(log_file)
 logging.config.dictConfig(LOGGING_CONFIG)
 
 # Create logger
-logger = logging.getLogger(__name__)
-logger.info('Beginning Dashboard Session')
+# logger = logging.getLogger(__name__)
+# logger.info('Beginning Dashboard Session')
 
 def _execute_insert(self, conn, keys, data_iter):
     #print "Using monkey-patched _execute_insert"
@@ -79,6 +80,7 @@ class DashboardSession(object):
     # Instantiate globals as class variables
     dashboard_dir = ccr.file_project
     df_keys = all_df_keys
+    __version__ = __version__
 
     print('Pulling diagnostic comments from Smartsheets...')
     # Smartsheet comments from previous reporting months. I'm going to hardcode Jan 2021 as the earliest date (column index 69 - Nice) but we can change that easily
@@ -113,7 +115,7 @@ class DashboardSession(object):
     def __init__(self,
                  project_list=None,
                  data_cutoff_date=None,
-                 data_source='AE_data_',
+                 data_source=None,
                  **kwargs):
 
         print('Initializing DashboardSession...')
@@ -124,7 +126,7 @@ class DashboardSession(object):
 
         # Add projects to the session if provided
         if project_list:
-            if isinstance(project_list, (str, unicode, list)):
+            if isinstance(project_list, (str, unicode, Iterable)):
                 self.add_projects(project_list)
             else:
                 print('`project_list` must be a string or list. The projects provided were not added to the DashboardSession. Please try again in the initialized DashboardSession.')
@@ -139,10 +141,13 @@ class DashboardSession(object):
             self.data_cutoff_date = datetime.today()
 
         # Set data source
-        self.data_source = data_source
+        if data_source:
+            self.data_source = data_source
+        else:
+            self.data_source = f"{datetime.now().year}_data_"
             
         # Get snow dataframe
-        self.raw_snow_df = utils.get_snow_df(dashboard_folder, data_source, file_format='csv')
+        self.raw_snow_df, self.snow_file = utils.get_snow_df(dashboard_folder, self.data_source, file_format='csv')
 
         # Create list for projects that have errored out
         self.errored_projects = {}
@@ -160,7 +165,7 @@ class DashboardSession(object):
         self.data_cutoff_date: datetime
         """Last day to process data."""
 
-    def __str__(self):
+    def __repr__(self):
         return 'DashboardSession object with {} projects'.format(len(self.project_list))
 
 
@@ -177,21 +182,26 @@ class DashboardSession(object):
             get_neighbors (bool, optional): Flag to add & initialize neighbor [Projects][ccrenew.dashboard.project.Project].
         """
         # Type validation
-        if not isinstance(project_name, (str, unicode, list)):
-            raise TypeError("`project_name` must be a string or Python list! Please reformat input and try again.")
+        if not isinstance(project_name, (str, unicode, Iterable, Project)):
+            raise TypeError("`project_name` must be a string, Project object, or Python list! Please reformat input and try again.")
 
         # Determine behavior based on what was passed to method
-        if isinstance(project_name, list):
+        if isinstance(project_name, Iterable) and not isinstance(project_name, str):
             self.add_projects(project_name)
         elif project_name in self.project_list:
-            return
+            project = self.project_list[project_name]
+        elif isinstance(project_name, Project):
+            project = project_name
+            self.project_list[project.project_name] = project
         else:
             try:
                 project = self.__initialize_project(project_name, get_neighbors=get_neighbors)
                 self.project_list[project.project_name] = project
             except RuntimeError as e:
                 print(e)
-                logger.warn(e)
+                # logger.warn(e)
+
+        return project
     
 
     def add_projects(self, project_list, get_neighbors=True):
@@ -201,15 +211,15 @@ class DashboardSession(object):
             project_list (list, str): list of project names to add & initialize to the [DashboardSession][ccrenew.dashboard.session.DashboardSession] instance.
             get_neighbors (bool, optional): See [add_project()][ccrenew.dashboard.session.DashboardSession.add_project].
         """
-        if isinstance(project_list, (str, unicode)):
+        if isinstance(project_list, (str, unicode, Project)):
             self.add_project(project_list, get_neighbors=get_neighbors)
 
-        elif isinstance(project_list, list):
+        elif isinstance(project_list, Iterable):
             for project_name in project_list:
                 self.add_project(project_name, get_neighbors=get_neighbors)
         
         else:
-            raise TypeError("`project_list` must be a string or Python list! Please reformat input and try again.")
+            raise TypeError("`project_list` must be a string, Project object, or Python list! Please reformat input and try again.")
                 
 
     def __initialize_project(self, project_name, get_neighbors=True):
@@ -230,30 +240,19 @@ class DashboardSession(object):
             # Log that project was not found
             warn_msg = '"{}" not found in df keys. Please verify the project is present in df keys & the spelling is correct and try again'.format(project_name)
             print(warn_msg)
-            logger.warn(warn_msg)
+            # logger.warn(warn_msg)
 
             # Remove project from project_list since we won't be able to initialize it
             del self.project_list[project_name]
             return
 
+        # Get neighbor projects
         if get_neighbors:
-            # Find all neighbors that satisfy the distance and equipment requirements
-            neighbor_list = neighbs.find_nearby_similar_projects(project_name, print_data=False, include_retired=True).index.tolist()
-            
-            # Then remove the search project from the list
-            # (Sometimes it will return an empty list if there is some data missing in `df_keys`) so we need to check if it returned a non-empty list first
-            try:
-                neighbor_list.remove(project_name)
-            except ValueError:
-                pass
-
-            # If the project has neighbors, we'll add them to the DashboardSession instance
-            # Set get_neighbors to False so we don't get neighbors of neighbors of neighbors etc.
-            if neighbor_list:
-                self.add_projects(neighbor_list, get_neighbors=False)
+            neighbor_list = self.__get_neighbors(project_name)
         else:
-            neighbor_list = None
+            neighbor_list = {}
 
+        # Set battery/tracker attributes
         Battery_AC_site = project_name in self.battery_ac_sites
         Battery_DC_site = project_name in self.battery_dc_sites
         Tracker_site = project_name in self.tracker_sites
@@ -269,10 +268,40 @@ class DashboardSession(object):
         proj_init_dict['Battery_DC_site'] = Battery_DC_site
         proj_init_dict['Tracker_site'] = Tracker_site
         proj_init_dict['raw_snow_df'] = self.raw_snow_df
+        proj_init_dict['snow_file'] = self.snow_file
         proj_init_dict['neighbor_list'] = neighbor_list
 
         project = Project(proj_init_dict)
         return project
+    
+
+    def __get_neighbors(self, project_name):
+        # Find all neighbors that satisfy the distance and equipment requirements
+        all_neighbors = neighbs.find_nearby_projects(project_name, print_data=False, include_retired=False).index.tolist()
+        poa_neighbors = neighbs.find_nearby_similar_projects(project_name, print_data=False, include_retired=False).index.tolist()
+        
+        # Then remove the search project from the list
+        # (Sometimes it will return an empty list if there is some data missing in `df_keys`) so we need to check if it returned a non-empty list first
+        try:
+            all_neighbors.remove(project_name)
+        except ValueError:
+            pass
+        try:
+            poa_neighbors.remove(project_name)
+        except ValueError:
+            pass
+
+        # We'll create a neighbor_list dictionary
+        # Value=True: we can use it for POA data sub
+        # Value=False: we can only use it for weather data sub
+        neighbor_list = {project_name: True if project_name in poa_neighbors else False for project_name in all_neighbors}
+
+        # If the project has neighbors, we'll add them to the DashboardSession instance
+        # Set get_neighbors to False so we don't get neighbors of neighbors of neighbors etc.
+        if neighbor_list:
+            self.add_projects(neighbor_list, get_neighbors=False)
+        
+        return neighbor_list
 
 
     def get_project(self, project_name: str, get_neighbors: bool=True) -> Project:
@@ -296,7 +325,7 @@ class DashboardSession(object):
         except KeyError:
             self.add_project(project_name, get_neighbors=get_neighbors)
             project = self.project_list[project_name]
-        
+ 
         return project
 
 
@@ -313,25 +342,29 @@ class DashboardSession(object):
         Returns:
             Processed Project object.
         """
-        if isinstance(project_name, list):
-            # Pass function arguments to `process_projects()`.
-            # Argument 0 is `self` so we'll pass arguments 1 and on
-            func_args = list(locals().values())[1:]
-            project = self.process_projects(*func_args)
-
+        if isinstance(project_name, Iterable) and not isinstance(project_name, str):
+            # If we call this method with a list we'll default to series processing
+            project_list = project_name
+            project = self.process_projects(project_list=project_list, method='series', reprocess=reprocess)
+            
             return project
+
         elif isinstance(project_name, (str, unicode, Project)):
             start = time.time()
             print(f'Processing {project_name}....')
             # Get project object
-            project = self.get_project(project_name)
+            try:
+                project = self.get_project(project_name)
+            except Exception as e:
+                print(e)
+                return
 
             # Pull the last updated dates from the filesystem for the config & Powertrack files
             try:
                 last_update_config_file = os.path.getmtime(project.config_filepath)
                 last_update_powertrack_file = os.path.getmtime(project.powertrack_filepath)
             except:
-                self.__update_project_filepaths()
+                self.__update_project_filepaths(project)
                 last_update_config_file = os.path.getmtime(project.config_filepath)
                 last_update_powertrack_file = os.path.getmtime(project.powertrack_filepath)
 
@@ -350,7 +383,20 @@ class DashboardSession(object):
                     return project
 
             # Process data - update neighbor sensors first then process the project
+            project.neighbor_list = self.__get_neighbors(project_name)
             self.__get_neighbor_sensor_data(project)
+            for neighbor_name in project.neighbor_list:
+                neighbor = self.project_list[neighbor_name]
+                if neighbor.df.empty:
+                    neighbor.load_production_data()
+                if any([neighbor.df_bool.empty, neighbor.df_sensors.empty]):
+                    neighbor.generate_bool()
+                if neighbor.df_sensors_native_avg.empty:
+                    neighbor._calculate_native_sensor_averages()
+            neighbor_sensors = {self.project_list[neighbor].project_name: self.project_list[neighbor].df_sensors_native_avg for neighbor in project.neighbor_list}
+            project.generate_bool()
+            project._calculate_native_sensor_averages()
+            # project.calculate_sensor_average_data_sub(neighbor_sensors, get_solcast=True)
             try:
                 project._process_data()
             except Exception:
@@ -374,52 +420,93 @@ class DashboardSession(object):
             raise TypeError("`project_name` must be a string or Project object! Please reformat input and try again.")
     
     
-    def process_projects(self, project_list, reprocess=False):
+    def process_projects(self, project_list, method='series', reprocess=False):
         """Calls [process_project()][ccrenew.dashboard.session.DashboardSession.process_project] for each
         [Project][ccrenew.dashboard.project.Project] in a list.
 
         Args:
             project_list (list): List of [Project][ccrenew.dashboard.project.Project] names process.
-            reprocess (bool, optional): See [process_project()][ccrenew.dashboard.session.DashboardSession.process_project].
+            method (str): Option to perform processing operations in series or parallel.
+            reprocess (bool): See [process_project()][ccrenew.dashboard.session.DashboardSession.process_project].
         """
-        # Store function arguments in case we need to pass them to `process_project()`.
-        # Argument 0 is `self` so we'll pass arguments 1 and on
-        func_args = list(locals().values())[1:]
-
         if isinstance(project_list, (str, unicode, Project)):
-            project = self.process_project(*func_args)
+            project_name = project_list
+            project = self.process_project(project_name, reprocess)
 
             return project
-        elif isinstance(project_list, list):
-            # If a list was passed, argument 0 is the project list
-            # So we'll pass arguments 1 and on
-            func_args = func_args[1:]
-
+        elif isinstance(project_list, Iterable):
             if len(project_list) == 1:
                 project_name = project_list[0]
-                project = self.process_project(project_name, *func_args)
+                project = self.process_project(project_name, reprocess)
 
                 return project
             else:
-                for i, project_name in enumerate(project_list):
-                    print('*******************************************************')
-                    print(f'Processing {project_name} - Project {i+1} of {len(project_list)}.')
-                    print('*******************************************************')
-                    self.process_project(project_name, *func_args)
+                if method == 'series':
+                    start = time.time()
+                    for i, project_name in enumerate(project_list):
+                        project = self.process_project(project_name, reprocess)
+                        if not project:
+                            continue
+
+                        elapsed = time.time() - start
+                        print('*******************************************************')
+                        print(f'{project_name} complete. {i+1} of {len(project_list)} projects completed. Total elapsed time: {elapsed:2f}s')
+                        print('*******************************************************')
+
+                elif __name__ == 'ccrenew.dashboard.session':
+                    start = time.time()
+
+                    # We'll use half of the available processors for the machine to not completely bog it down.
+                    max_workers = int(os.cpu_count()/2)
+                    print(f"Processing {len(project_list)} projects with {min(max_workers, len(project_list))} workers.")
+                    print("Progress messages from workers will be suppressed in an interactive session.")
+                    print("Projects will report success upon completion and may not complete in the order in which they were submitted.")
+
+                    # Construct a dict of kwargs for each project to pass to `process_pool`
+                    kwargslist =  [{"project_name":project_name,
+                                    "session":DashboardSession,
+                                    "data_source": self.data_source,
+                                    "reprocess":reprocess,
+                                    "project_num":i+1,
+                                    "project_total":len(project_list),
+                                    "batch_start":start
+                                    } for i, project_name in enumerate(project_list)]
+
+                    # Results are a list of Future objects, where we can get the results of the function call from `.result()`
+                    results = process_pool(kwargslist)
+                    for result in results:
+                        processed_project_list = result.result()['project_list']
+                        for project_dict in processed_project_list:
+                            try:
+                                project_name, project = project_dict
+                                # If the project is already processed in the parent session we won't add it
+                                if project_name in self.project_list and self.project_list[project_name].processed:
+                                    pass
+                                else:
+                                    self.project_list[project_name] = project
+                            except:
+                                continue
+
+                elapsed = time.time() - start
+                print(f"\n{len(project_list)} projects processed. Elapsed time: {elapsed:.2f}s.\n")
         else:
             raise TypeError("`project_list` must be a string or Python list! Please reformat input and try again.")
+
+
+    def process_all_projects(self, method='parallel', reprocess=True):
+        """
+        Processes all projects in the [project_list][ccrenew.dashboard.session.DashboardSession.project_list]
+        for the [DashboardSession][ccrenew.dashboard.session.DashboardSession] instance.
+
+        Args:
+            method (str): Option to perform operations in series or parallel.
+            reprocess (bool): See [process_project()][ccrenew.dashboard.session.DashboardSession.process_project].
+        """
+        project_list = list(self.project_list.keys())
+        self.process_projects(project_list, method=method, reprocess=reprocess)
     
 
-    def process_all_projects(self):
-        """Processes all projects in the [project_list][ccrenew.dashboard.session.DashboardSession.project_list]
-        for the [DashboardSession][ccrenew.dashboard.session.DashboardSession] instance."""
-        # FIXME: add data cutoff date in here in case you want to change that in the middle of a session
-        # FIXME: Same with process project object below
-        project_list = list(self.project_list.keys())
-        self.process_projects(project_list)
-
-
-    def export_project(self, project_name: str, server: str='dev', save_pickle: bool=True, *args) -> None:
+    def export_project(self, project_name: str, server: str='dev', save_pickle: bool=True, dest: Union[str, list]='all') -> None:
         """
         Export project to desired location(s).
 
@@ -427,38 +514,38 @@ class DashboardSession(object):
             project_name (str or Project): Name of a project to export.
             server (str): SQL table to export to. Options are `prod` or `dev`.
             save_pickle (bool): Option to save pickle after exporting.
-        
-        Optional Args:
-            These allow the user to export only certain parts of the project.
-            
-            <b>Options:</b>
-            <br>
-                <code>'excel'</code>: Exports dataframes to Excel dashboard template.
-            <br>
-                <code>'df'</code>: Export dataframe to the `Dataframes` folder of the project's
-                [project_directory][ccrenew.dashboard.project.Project.project_directory],
-                S3, and Bartertown PostgreSQL database.
-            <br>
-                <code>'summary'</code>: Exports monthly summary data to S3 and
-                Bartertown PostgreSQL database.
-            <br>
-                <code>'snow'</code>: Exports [snow_data][ccrenew.dashboard.project.Project.snow_data] and
-                [snow_coverage][ccrenew.dashboard.project.Project.snow_coverage] dataframes to S3.
+            dest (str or list): Option to export only certain parts of the project. `all` will export to all of the below destinations.
+
+        `dest` Options
+
+        | dest      | Description                                                              |
+        |-----------|--------------------------------------------------------------------------|
+        |  `excel`  | Export dataframes to Excel dashboard template.                          |
+        |   `df`    | Export dataframe to the `Dataframes` folder of the project's [project_directory][ccrenew.dashboard.project.Project.project_directory], S3, and Bartertown PostgreSQL database. |
+        | `summary` | Export monthly summary data to S3 and Bartertown PostgreSQL database.   |
+        |  `snow`   | Export [snow_data][ccrenew.dashboard.project.Project.snow_data] and [snow_coverage][ccrenew.dashboard.project.Project.snow_coverage] dataframes to S3. |
+        |  `bool`   | Export bool file to the `Plant_Config_File` directory of the project's [project_directory][ccrenew.dashboard.project.Project.project_directory]. |
         """
-        if isinstance(project_name, list):
-            # Pass function arguments to `export_projects()`.
-            # Argument 0 is `self` so we'll pass arguments 1 and on
-            func_args = list(locals().values())[1:]
-            project = self.export_projects(func_args)
+        # Pass function arguments to `export_projects()`.
+        # Remove `self`, i.e. the DashboardSession instance
+        func_args = locals().copy()
+        func_args.pop('self')
+
+        if isinstance(project_name, Iterable) and not isinstance(project_name, str):
+            # Rename project_name to project_list for `export_projects`
+            func_args.pop('project_name')
+            func_args['project_list'] = project_name
+            self.export_projects(**func_args)
         elif isinstance(project_name, (str, Project)):
             project = self.process_project(project_name)
-            project.export(server, save_pickle, *args)
+            func_args.pop('project_name')
+            project.export(**func_args)
             print(f"{project.project_name} successfully exported.")
         else:
             raise TypeError("`project_name` must be a string or Project object! Please reformat input and try again.")
 
     
-    def export_projects(self, project_list: str, server: str='dev', save_pickle: bool=True, *args) -> None:
+    def export_projects(self, project_list: str, server: str='dev', save_pickle: bool=True, dest: Union[str, list]='all', method: str='series') -> None:
         """Calls [export_project()][ccrenew.dashboard.session.DashboardSession.export_project] for each
         [Project][ccrenew.dashboard.project.Project] in a list.
 
@@ -466,27 +553,75 @@ class DashboardSession(object):
             project_list (list): List of [Project][ccrenew.dashboard.project.Project] names process.
             server (str): See [export_project()][ccrenew.dashboard.session.DashboardSession.export_project].
             save_pickle (bool): See [export_project()][ccrenew.dashboard.session.DashboardSession.export_project].
-        
-        Optional Args:
-            See [export_project()][ccrenew.dashboard.session.DashboardSession.export_project].
+            dest (str or list): See [export_project()][ccrenew.dashboard.session.DashboardSession.export_project].
+            method(str): Option to export in series or parallel.
         """
         # Store function arguments in case we need to pass them to `export_project()`.
         # Argument 0 is `self` so we'll pass arguments 1 and on
-        func_args = list(locals().values())[1:]
+        func_args = locals().copy()
+        func_args.pop('self')
+        method = func_args.pop('method')
 
         if isinstance(project_list, (str, unicode, Project)):
-            project = self.export_project(*func_args)
-        elif isinstance(project_list, list):
-            # If a list was passed, argument 0 is the project list
-            # So we'll pass arguments 1 and on
-            func_args = func_args[1:]
+            # Rename project_list to project_name for `export_project`
+            func_args.pop('project_list')
+            func_args['project_name'] = project_list
+            self.export_project(**func_args)
+        elif isinstance(project_list, Iterable):
+            # If a list was passed, we'll pop the project list & use the remaining args
+            project_list = func_args.pop('project_list')
 
             if len(project_list) == 1:
                 project_name = project_list[0]
-                self.export_project(project_name, *func_args)
+                self.export_project(project_name, **func_args)
             else:
-                for project_name in project_list:
-                    self.export_project(project_name, *func_args)
+                if method == 'series':
+                    start = time.time()
+                    for i, project_name in enumerate(project_list):
+                        self.export_project(project_name, **func_args)
+                    
+                        elapsed = time.time() - start
+                        print('*******************************************************')
+                        print(f'{project_name} complete. {i+1} of {len(project_list)} projects exported. Total elapsed time: {elapsed:2f}s')
+                        print('*******************************************************')
+                
+                elif __name__ == 'ccrenew.dashboard.session':
+                    start = time.time()
+
+                    # We'll use half of the available processors for the machine to not completely bog it down.
+                    max_workers = int(os.cpu_count()/2)
+                    print(f"Exporting {len(project_list)} projects with {min(max_workers, len(project_list))} workers.")
+                    print("Progress messages from workers will be suppressed in an interactive session.")
+                    print("Projects will report success upon completion and may not complete in the order in which they were submitted.")
+
+                    # Construct a dict of kwargs for each project to pass to `process_pool`
+                    kwargslist =  [{"project_name":project_name,
+                                    "session":DashboardSession,
+                                    "data_source": self.data_source,
+                                    "func_args":func_args,
+                                    "project_num":i+1,
+                                    "project_total":len(project_list),
+                                    "batch_start":start
+                                    } for i, project_name in enumerate(project_list)]
+
+                    # Results are a list of Future objects, where we can get the results of the function call from `.result()`
+                    results = export_pool(kwargslist)
+                    for result in results:
+                        exported_project_list = result.result()['project_list']
+                        for project_dict in exported_project_list:
+                            try:
+                                project_name, project = project_dict
+                                # If the project is already processed in the parent session we won't add it
+                                if project_name in self.project_list and self.project_list[project_name].processed:
+                                    pass
+                                else:
+                                    self.project_list[project_name] = project
+                            except:
+                                continue
+
+                elapsed = time.time() - start
+                print(f"\n{len(project_list)} projects exported. Elapsed time: {elapsed:.2f}s.\n")                        
+
         else:
             raise TypeError("`project_list` must be a string or Python list! Please reformat input and try again.")
 
@@ -495,7 +630,7 @@ class DashboardSession(object):
         """Remove the selected project from the active session.
 
         Args:
-            project_name (str): The name of the [Project][ccrenew.dashboad.project.Project]
+            project_name (str): The name of the [Project][ccrenew.dashboard.project.Project]
                 to remove from the [DashboardSession][ccrenew.dashboard.session.DashboardSession] instance.
         """
         del self.project_list[project_name]
@@ -524,7 +659,7 @@ class DashboardSession(object):
         Raises:
             TypeError: if the wrong type is supplied to `project_name`.
         """
-        if isinstance(project_name, list):
+        if isinstance(project_name, Iterable) and not isinstance(project_name, str):
             self.save_pickles(project_name, store_plots=store_plots)
         elif isinstance(project_name, (str, Project)):
             project = self.process_project(project_name)
@@ -544,7 +679,7 @@ class DashboardSession(object):
         if isinstance(project_list, (str, Project)):
             project_name = project_list
             self.save_pickle(project_name, store_plots=store_plots)
-        elif isinstance(project_list, list):
+        elif isinstance(project_list, Iterable):
             for i, project_name in enumerate(project_list):
                 print('*******************************************************')
                 print(f'Pickling {project_name} - Project {i+1} of {len(project_list)}.')
@@ -579,22 +714,21 @@ class DashboardSession(object):
 
         Args:
             project_name (str): The name of the [Project][ccrenew.dashboard.project.Project] to load.
-            year (int, optional): Year to load for the pickle. Defaults to None, which pulls the pickle for the current year.
+            year (int, optional): Deprecated. Year for the pickle will always load the year to match the session.
             data_cutoff_date (str or datetime): Cutoff date for the [Project][ccrenew.dashboard.project.Project]. When
                 loading a project that was pickled some time ago the [data_cutoff_date][ccrenew.dashboard.session.DashboardSession.data_cutoff_date]
                 could prevent the data from loading to the current date. Default is None,
                 which will reset the [data_cutoff_date][ccrenew.dashboard.session.DashboardSession.data_cutoff_date] to the current day.
             show_plots (bool, optional): Option to show plots if pickle was stored with plots saved.
         """
-        if not year:
-            year = datetime.now().year
+        year = self.data_source.split('_')[0]
         try:
             project_directory = self.df_keys.query("Project == @project_name")['Folder'].item()
         except IndexError:
             # Log that project was not found
             warn_msg = '"{}" not found in df keys. Please verify the project is present in df keys & the spelling is correct and try again'.format(project_name)
             print(warn_msg)
-            logger.warn(warn_msg)
+            # logger.warn(warn_msg)
             return
 
         pickle_jar = os.path.join(self.dashboard_dir,
@@ -603,12 +737,19 @@ class DashboardSession(object):
                                   'Pickle_Jar')
         
         pickle_name = utils.picklefy_project_name(project_name)
-        pickle_file = str(year) + "_" + pickle_name + ".pickle"
+        pickle_file = year + "_" + pickle_name + ".pickle"
         pickle_path = os.path.join(pickle_jar, pickle_file)
         
         with open(pickle_path, 'rb') as f:
             project = pickle.load(f)
 
+        # Check if project version matches session version & re-process if not
+        try:
+            if project.__version__ != self.__version__:
+                project = self.process_project(project_name)
+        except:
+            project = self.process_project(project_name)
+
         # Remove plotter object if show_plots=False
         if not show_plots and project.plotter:
             project.plotter.close_plots()
@@ -645,7 +786,7 @@ class DashboardSession(object):
             project_name = project_list
             self.load_pickle(project_name, year=year, data_cutoff_date=data_cutoff_date, show_plots=show_plots)
             return
-        elif isinstance(project_list, list):
+        elif isinstance(project_list, Iterable):
             for project_name in project_list:
                 try:
                     project = self.load_pickle(project_name, year=year, data_cutoff_date=data_cutoff_date, show_plots=show_plots)
@@ -733,7 +874,7 @@ class DashboardSession(object):
                     self.__draw_plots(project, plot_order=plot_order, *args, **kwargs)
             else:
                 self.__draw_plots(project, plot_order=plot_order, *args, **kwargs)
-        elif isinstance(project_name, list):
+        elif isinstance(project_name, Iterable):
             project_list = project_name
             project_count = len(project_list)
             for i, project_name in enumerate(project_list):
@@ -776,34 +917,17 @@ class DashboardSession(object):
         Returns:
             Project: a `ccrenew.dashboard.project.Project` with updated filepaths
         """
-        
-        # Update filepath references for the project
-        project.dashboard_dir = self.dashboard_dir
-
         # Build filename for config file
-        config_filename = project.project_name + r'_Plant_Config_MACRO.xlsm'
-        project.config_filepath = os.path.join(project.dashboard_dir,
-                                        project.project_directory,
-                                        project.project_name,
-                                       'Plant_Config_File',
-                                        config_filename)
+        project.config_filepath = project._find_config_file()     
 
-        # Find Powertrack file
-        project.data_AE_dir = os.path.join(project.dashboard_dir,
-                                        project.project_directory,
-                                        project.project_name,
-                                        'Powertrack_data')
-        project.data_AE_all_files = [f for f in os.listdir(project.data_AE_dir) if os.path.isfile(os.path.join(project.data_AE_dir, f))]
-        data_AE = [s for s in project.data_AE_all_files if project.data_source in s]
+        # Build filename for bool file
+        project.bool_filepath = project._find_bool_file()     
 
         # Build filename for Powertrack file
-        project.powertrack_filepath = os.path.join(project.dashboard_dir, project.project_directory, project.project_name, 'Powertrack_data', data_AE[0])
+        project.powertrack_filepath = project._find_powertrack_file()
 
         # Check for pickle jar folder & create if it doesn't exist
-        project.pickle_jar = os.path.join(project.dashboard_dir,
-                                       project.project_directory,
-                                       project.project_name,
-                                       'Pickle_Jar')
+        project.pickle_jar = project._find_pickle_jar() 
         
         return project
     
@@ -839,7 +963,10 @@ class DashboardSession(object):
             try:
                 neighbor = self.get_project(neighbor_name, get_neighbors=False)
             except KeyError:
-                print('Neighbor for {} not found: {}. Sensor data for the neighbor will be blank. Check logs to determine why neighbor was not added to DashboardSession'.format(project.project_name, neighbor_name))
+                print(f'Neighbor for {project.project_name} not found: {neighbor_name}. Sensor data for the neighbor will be blank. Check logs to determine why neighbor was not added to DashboardSession')
+                continue
+            except Exception as e:
+                print(f'Neighbor for {project.project_name} errored: {neighbor_name}. Sensor data for the neighbor will be blank. Check logs to determine why neighbor was not added to DashboardSession.\nError details: {e}')
                 continue
             
             # Reload neighbor's config or powertrack files if needed
