@@ -95,7 +95,7 @@ def get_snow_df(dashboard_folder, data_source, file_format):
         snow_file = os.path.join(dashboard_folder, 'Python_Functions', 'Snow Losses', '{}Operating-States-snowfall.csv'.format(data_source))
         raw_snow_df = read_csv(snow_file, index_col=0)
 
-    return raw_snow_df
+    return raw_snow_df, snow_file
 
 def retrieve_s3_df(bucket, key):
 # helper function to get data from s3
